/* 
 * File:   GetTemps.c
 * Author: Ken
 */

#define _SUPPRESS_PLIB_WARNING
#include <plib.h>
#include "../cfunctionsV52.h"   // Version 5.2 of CFunctions.h
#include <p32xxxx.h>

#define INITIALISE (0)          // Initialise = 0, GetTemp = 1
#define INSIDE (0)              // Inside temperature probe
#define EIGHT (8)
#define OneWirePin (PORTBbits.RB10)


void DQLo (int, int);
void DQHi (int);
int RESETT (void);
void MatchROM (unsigned long long int);
void SendROMCode (unsigned long long int);
void TxByte (void);
void SendCode (short int);
void SendCode4E (void);
void GetTemp(unsigned long long int);
void RxByte (void);
void ReadTemp (void);
void ConvertTemp (char []);

unsigned short IOByte;
unsigned long long ROMCode;

// CONVERTTEMP
void ConvertTemp (char Deg [])  // Converts Temp value from DS18B20 format to 
{                               // a string
 short int Tmp;
  Deg [1] = ' ';
 
  if ((IOByte & 0x8000) == 0x8000)
    {
      Deg [1] = '-';
      IOByte = IOByte ^ 0xFFFF;
      IOByte = 1 + IOByte;
    }
  IOByte = IOByte & 0x7FF;
  Tmp = IOByte & 0xE;
  if (Tmp == 0)
    Deg [5] = '0';
  else if (Tmp == 2)
    Deg [5] = '1';
  else if (Tmp == 4)
    Deg [5] = '3';
  else if (Tmp == 6)
    Deg [5] = '4';      
  else if (Tmp == 8)
    Deg [5] = '5';
  else if (Tmp == 0xA)
    Deg [5] = '6';
  else if (Tmp == 0xC)
    Deg [5] = '8';
  else 
    Deg [5] = '9';
    
  Tmp = IOByte >> 4;
  Deg [2]= (Tmp / 10) + '0';
  Deg [3]= (Tmp % 10) + '0';
  if (Deg [2] == '0')
    Deg [2] = ' ';
  
  if ((Deg [1] == '-') && (Deg [2] == ' '))
    {
      Deg [1] = ' ';
      Deg [2] = '-';
    }
}

// READTEMP
void ReadTemp (void)
{
int Count = 0;
 
  IOByte = 0;
  do
    {
      RxByte();
    } while ((Count = Count + 1) < 2);   
  RESETT();
}

//RXBYTE
void RxByte (void)
{
int BitCount = EIGHT;
short int Tmp = 0;

  do
    {  
      IOByte = IOByte >> 1;  
      DQLo(6, 3);
      DQHi(3);
      uSec(7);
      
      Tmp = (PORTB & 0x0400);
      if (Tmp != 0) 
          IOByte = 0x8000 | IOByte;  
      uSec (55);
     } while ((BitCount = BitCount - 1) != 0);  
}

// GETTEMP
void GetTemp(unsigned long long int Code)
{
  int Count;
  SendCode (0x44);              // CONVERT TEMP Command
  Count = 0;
  do
    {
    IOByte = 0;  
    RxByte(); 
    Count = Count + 1;
    } while (IOByte != 0xFF00); 
   
  RESETT();
  SendCode(0x55);               // MATCH ROM Command 
  SendROMCode(Code);
  SendCode(0xBE);               // READ SCRATHPAD Command
  ReadTemp();
}

// TXBYTE
void TxByte (void)
{
int BitCount = EIGHT;
  do
    {
      DQLo(6, 3);
      uSec(3);
      if (IOByte & 0x0001 == 1)
        DQHi(3);
      uSec(60);
      DQHi(3);
      IOByte = IOByte >> 1;
      uSec(5);
    } while ((BitCount = BitCount - 1) != 0);
}

// SENDCODE4E
void SendCode4E (void)              
{
  SendCode (0X4E);                  // WRITE SCRATHPAD Command
  SendCode (0);                     // Send INIT data to Th register
  SendCode (0);                     // Send INIT data to Tl register
  SendCode (0b01011111);            // Send '10-bit Temperature Resolution' code
                                    // to Configuration Register                   
}

// SENDCODE
void SendCode (short int Cmd)       // Send DS18B20 Commands
{
  IOByte = Cmd; 
  TxByte();
}

// SENDROMCODE
void SendROMCode (unsigned long long int Code)      // Send MATCH ROM Code
{
 int ByteCount = EIGHT;
 unsigned long long int Tmp;
 
 Tmp = Code;
  do
    {
      IOByte = (short int) Tmp & 0xFF;
      TxByte ();
      Tmp = Tmp >> 8;
    } while ((ByteCount = ByteCount - 1) != 0);
}

// MATCHROM
void MatchROM (unsigned long long int Code)
{
  SendCode(0x55); 
  SendROMCode(Code);
}

// RESETT
int RESETT (void)
{
  int PDByte;
  
  PDByte = 0;
  DQLo(6, 3);
  uSec(500);
  DQHi(3);
  uSec(70);     
  if (OneWirePin != FALSE) 
  {
    PDByte = 1;
  }
  uSec(440);  
  return (PDByte);
}

// DQHI
void DQHi (int t)
  {  
    TRISBSET  = 0x0400;
    uSec (t);
  }

// DQLO
void DQLo (int t1, int t2)
{
    uSec (t1);
    TRISBCLR = 0x0400;
    uSec (t2);
}

void main(long long * Mode, long long * Probe, char Temper [], long long * Error)
{
  int Count = 2;
  unsigned long long TempIn  = 0xE000000522434228;
  unsigned long long TempOut = 0xE000000522434228;
  
    LATBCLR = 0x0400;
       
    if (*Mode == INITIALISE)        // Init Ext & Int Temp probes, ie. 10 bit
      {
        ROMCode = TempIn;
        do
          {
            if((*Error = RESETT()) != FALSE)      // Get error, if any
              {
                // Need more code to report error
                *Error = 100;
              }
            uSec(300);
            MatchROM (ROMCode);
            SendCode4E();
            ROMCode = TempOut;
          } while ((Count = Count - 1) != 0);   
      }  
    else                              // else Read a Temp Probe
      {
        if (*Probe == INSIDE)
          ROMCode = TempIn;
        else
          ROMCode = TempOut;
        RESETT();
        MatchROM (ROMCode);
        SendCode(0x44);
        GetTemp (ROMCode);
        ConvertTemp (Temper);
      } 
/*    Temper [0] = 'A';
    Temper [1] = 'B';
    Temper [2] = 'C';
    Temper [3] = 'D';
    Temper [4] = 'E';
    Temper [5] = 'F';
    Temper [6] = '0x60';
    Temper [7] = 'H';
    Temper [8] = 'I';
    Temper [6] = 0x60;*/
}
